
# q-expansion of newform 960.4.f.h, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 960
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[511, [1, 0]], [901, [1, 0]], [641, [1, 0]], [577, [-1, 0]]]
aps_data = [[0, 0], [0, -3], [10, -5], [0, 18], [-30, 0], [0, -38], [0, 70], [12, 0], [0, 72], [-64, 0], [-312, 0], [0, 138], [-374, 0], [0, 468], [0, -132], [0, -446], [510, 0], [-754, 0], [0, 384], [-924, 0], [0, -340], [-72, 0], [0, 156], [290, 0], [0, 376], [-104, 0], [0, 1014], [0, 1284], [570, 0], [0, 254], [0, 138], [894, 0], [0, 3050], [-2352, 0], [2476, 0], [-3192, 0], [0, -1218], [0, 2328], [0, 2928], [0, 502], [-4026, 0], [-594, 0], [1044, 0], [0, -3252], [0, -2914], [4872, 0], [-1872, 0], [0, 1302], [0, -708], [-5082, 0], [0, 5786], [3156, 0], [-3306, 0], [4134, 0], [0, 3482], [0, -2856], [6464, 0], [-5184, 0], [0, 3818], [6094, 0], [0, -4440], [0, -5950], [0, 924], [-7020, 0], [0, 5244], [0, 5194], [11628, 0], [0, -4372], [0, -12564], [970, 0], [0, -1958], [1920, 0], [0, -8886], [0, -7470], [6312, 0], [0, -12876], [-8780, 0], [0, -3246], [1870, 0], [-15990, 0], [-3054, 0], [1194, 0], [-15264, 0], [0, 8220], [-16248, 0], [0, -8220], [9490, 0], [0, -5776], [-2812, 0], [0, -13710], [0, 6252], [7908, 0], [0, -17562], [13062, 0], [8556, 0], [0, 14436], [9536, 0], [-1130, 0], [0, -4104], [-6810, 0], [0, -23772], [0, -10762], [0, -1716], [-6670, 0], [5580, 0], [0, 17808], [0, -23004], [0, -3266], [2208, 0], [-21166, 0], [0, -17610], [0, -29610], [0, 2294], [-19704, 0], [4392, 0], [19586, 0], [0, -624], [0, 8556], [0, 22910], [-7590, 0], [-31970, 0], [0, 24996], [0, 3226], [0, 23412], [-16572, 0], [448, 0], [-11922, 0], [-4344, 0], [0, 26934], [0, -35962], [-4860, 0], [0, 29844], [1248, 0], [0, -25094], [23294, 0], [254, 0], [0, 21886], [0, 14904], [0, 23834], [31502, 0], [-21072, 0], [38072, 0], [0, 31578], [0, -41532], [24786, 0], [-1608, 0], [0, -8514], [0, 29726], [2136, 0], [0, 11928], [0, 27006], [29318, 0], [0, 7008], [0, -15024], [0, -1380], [17952, 0], [-19056, 0], [-26398, 0], [0, 21192], [-7300, 0], [0, 33540], [0, -32714], [0, -35334], [17094, 0], [0, 46814], [0, -5076], [-9456, 0], [0, 33486]]
